from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.formats import number_format

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.i18n.models import TranslationModel, I18NModel
from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE

from jmb.product.models import Product
from jmb.organization.models import Organization
from jmb.currency.models import Currency
from jmb.vat.models import Vat

class PriceList(DateModel, UserModel, StatusModel, OrderedModel):
    date_begin = models.DateField(
        db_index=True,
        verbose_name = _("date begin"),
        help_text = _('when this pricelist begins'),
    )
    date_end = models.DateField(
        db_index=True,
        verbose_name = _("date end"),
        help_text = _('when this pricelist ends'),
    )
    code = models.CharField(
        unique = True,
        max_length = 15,
        verbose_name = _("code"),
    )
    currency = models.ForeignKey (
        Currency,
        blank = True, null = True,
        related_name = "pricelists",
        db_index = True,
        verbose_name = _("Currency"),
    )
    pricelist_type = models.CharField (
        choices = settings.PRODUCT_PRICELIST_TYPE,
        max_length = 2,
        verbose_name = _("Price List Type"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        verbose_name=_("name"),
    )
    slug = models.SlugField(
        db_index=True,
        max_length = 200,
        verbose_name = _("Slug"),
        help_text = _("Used for URLs, auto-generated from name if blank"),
    )

    class Meta:
        verbose_name = _("price list")
        verbose_name_plural = _("price lists")
        ordering = ('ordering',)
        permissions = (
            ("view_pricelist", "Can view Price List"),
        )

    def __unicode__(self):
        return u"%s %s" % (self.code, self.name)

class PriceListItem(DateModel, UserModel, StatusModel, OrderedModel):
    price_list = models.ForeignKey (
        PriceList,
        related_name='prices_lists',
        verbose_name=_("price list"),
    )
    product = models.ForeignKey (
        Product,
        blank=True,
        null=True,
        related_name='prices_lists',
        verbose_name=_("product"),
    )
    date_begin = models.DateField(
        null=True,
        blank=True,
        verbose_name=_("date begin"),
    )
    date_end = models.DateField(
        null=True,
        blank=True,
        verbose_name=_("date end"),
    )
    price = models.DecimalField(
        max_digits=11,
        decimal_places=4,
        default=0,
        verbose_name=_("price"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "prices_lists",
        db_index = True,
        verbose_name = _("Vat"),
    )
    price_tax_included = models.BooleanField(
        verbose_name=_("price tax included"),
    )
    discount1 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("discount 1 %")
    )
    discount2 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("discount 2 %"),
    )
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )
    min_quantity = models.IntegerField(
        blank=True,
        null=True,
        verbose_name=_("min quantity"),
    )
    max_quantity = models.IntegerField(
        blank=True,
        null=True,
        verbose_name=_("max quantity"),
    )
    note = models.TextField (
        blank=True,
        null=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _("product price list item")
        verbose_name_plural = _("price list items")
        ordering = ('ordering',)
        permissions = (
            ("view_pricelistitem", "Can view Price List Item"),
        )

    def __unicode__(self):
        return u"%(ls_name)s - %(name)s (%(date_begin)s - %(date_end)s): %(price)s" % {
            'ls_name': self.price_list.name, 'name': self.price_list.name,
            'date_begin': self.date_begin, 'date_end': self.date_end, 'price': self.price
        }

    def get_date_begin(self):
        return self.date_begin or self.price_list.date_begin
    get_date_begin.short_description = _("Date Begin")

    def get_date_end(self):
        return self.date_end or self.price_list.date_end
    get_date_end.short_description = _("Date End")

    def get_price_vat_included(self):
        if self.price_tax_included:
            return self.price
        else:
            return self.price * (1 + (self.product.vat.aliquot_perc)/100)
    price_vat_included = property(get_price_vat_included)

    def get_price_vat_excluded(self):
        if self.price_tax_included:
            return self.price / (1 + (self.product.vat.aliquot_perc)/100)
        else:
            return self.price
    price_vat_excluded = property(get_price_vat_excluded)

    def get_last_price(self):
        last_price = (
            (self.price_vat_excluded * (1 - self.discount1)) *
            (1 - self.discount1)
        ) * (1 + self.recharge/100)
        return last_price
    last_price = property(get_last_price)

class OrganizationPriceList(DateModel, UserModel, StatusModel, OrderedModel):
    organization = models.ForeignKey(
        Organization,
        related_name='price_lists',
        verbose_name=_("Organization"),
    )
    price_list = models.ForeignKey(
        PriceList,
        blank=True, null=True,
        related_name='%(class)s_price_lists',
        verbose_name=_("Price List"),
    )
    discount1 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount 1 %"),
    )
    discount2 = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Discount 2 %"),
    )
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )

    class Meta:
        verbose_name = _("Organization Price List")
        verbose_name_plural = _("Organization Prices Lists")
        ordering = ('ordering',)
        unique_together = (("price_list", "organization",),)

    def __unicode__(self):
        price_list_name = ""
        if self.price_list:
            price_list_name = self.price_list.name
        return u"%s %s%% + %s%%" % (
            price_list_name,
            number_format(self.discount1, 1),
            number_format(self.discount2, 1)
        )
